const http = require('http')
const assert = require('assert')
const log = require('./log')


// []
const reverseProxy = function (options) {
	assert(Array.isArray(options.servers), 'Options.servers must be array')
	assert(options.servers.length > 0, 'Options.servers must more than 0')

	const servers = options.servers.map(str => {
		const s = str.split(':')
		return {
			hostname: s[0],
			port: s[1] || 80
		}
	})

	let i = 0
	const getTarget = function () {
		const t = servers[i]
		i++
		if(i >= servers.length) {
			i = 0
		}
		return t
	}
	
	const bindError = function (req, res, id) {
		return function (err) {
			const msg = String(err.stack || err)
			log(`${id} 发生错误: ${msg}`)
			if(!res.headersSent) {
				res.writeHead(500, {
					"content-type": "text/plain"
				})
				res.end()
			}
		}
	}
	
	const proxy = function (req, res) {
		const target = getTarget()
		const info = {
			...target,
			method: req.method,
			path: req.url,
			headers: req.headers
		}
		const id = `${req.method} ${req.url} => ${target.hostname}:${target.port}`
		log(`代理请求 ${id}`)

		const req2 = http.request(info, res2 => {
			res2.on('error', bindError(req, res, id))
			log(`${id} 响应`)
			res.writeHead(res2.statusCode, res2.headers)
			res2.pipe(res)
		})
		req.pipe(req2)
		req2.on('error', bindError(req, res, id))
	}

	return proxy
}

module.exports = reverseProxy